from ..params import titleStopwords
from .tools import check, checkPageRange, sequenceDistance

def comparePublicationSource(publiSource1,publiSource2) :

    if not publiSource1 or not publiSource2 :
        return 0

    else :
        d = []
        try :
            beginDate1 = str(publiSource1['begin']).lower().strip()
            beginDate2 = str(publiSource2['begin']).lower().strip()
        except ValueError as err :
            raise err

        try :
            title1 = str(publiSource1['title']).lower().strip()
            title2 = str(publiSource2['title']).lower().strip()
        except ValueError as err :
            raise err

        try :
            settlement1 = str(publiSource1['settlement']).lower().strip()
            settlement2 = str(publiSource2['settlement']).lower().strip()
        except ValueError as err :
            raise err

        d.append(check(beginDate1, beginDate2))
        d.append(check(settlement1, settlement2))
        d.append(check(title1, title2))

        if sum(d)>=1 :
            return 1

        elif -1 in d :
            return -1

        return 0

def compareIssn(issn1, issn2) :

    if isinstance(issn1, str) and isinstance(issn2, str) :
        issn1 = "".join(issn1.strip().lower().split("-"))
        issn2 = "".join(issn2.strip().lower().split("-"))
    else :
        return 0
    return check(issn1, issn2)


def compareId(doi1, doi2, nnt1, nnt2, pmId1, pmId2) :
    """
    Compare records identifier
    """

    if doi1 and doi2 :
        if isinstance(doi1, str) and isinstance(doi2, str) :
            if str(doi1).lower().strip() == str(doi2).lower().strip() :
                return 1
            else :
                return -1

    elif nnt1 and nnt2 :
        if isinstance(nnt1, str) and isinstance(nnt2, str) :
            if str(nnt1).lower().strip() == str(nnt2).lower().strip() :
                return 1
            else :
                return -1

    elif pmId1 and pmId2 :
        if isinstance(pmId1, str) and isinstance(pmId2, str) :
            if str(pmId1).lower().strip() == str(pmId2).lower().strip() :
                return 1
            else :
                return -1

    else :
        return 0

def comparePageRange(pageRange1,pageRange2) :
    """
    Compare pageRange
    """
    return checkPageRange(pageRange1, pageRange2)

def compareVolumaison(issue1, issue2, volume1, volume2) :
    """
    Compare volumaison
    """
    if issue1 and issue2 :
        if issue1== issue2 :
            return 1
        else :
            return -1

    elif volume1 and volume2 :
        if volume1 == volume2 :
            return 1
        else :
            return -1
    else :
        return 0

def comparePublisher(eissn1, eissn2, issn1, issn2, \
                       meeting1, meeting2, journal1, journal2,\
                       publiSource1, publiSource2,eisbn1, eisbn2, isbn1, isbn2) :
    """
    Compare publisher accornding to some record fields
    """
    eissn = compareIssn(eissn1, eissn2)
    issn = compareIssn(issn1, issn2)
    meeting = check(meeting1, meeting2)
    journal = check(journal1, journal2)
    settlement = comparePublicationSource(publiSource1, publiSource2)
    isbn = check(isbn1, isbn2)
    eisbn = check(eisbn1, eisbn2)

    if settlement or issn or eissn or meeting or journal or eisbn or isbn == 1 :
        return 1
    elif settlement or issn or eissn or meeting or journal or eisbn or isbn == -1:
        return -1
    else  :
        return 0

def compareTitle(titleDict1, titleDict2, threshold = .2) :
    """
    Compare title from title subfields which are : default, en, fr.
    :param titleDict : dictionary oo key value where key are 
    "en", "fr", "default" and values are titles.
    :return -1 (different), 0 (n/a) or 1 (identical)  
    """
    
    sortedKeys =  sorted(list(titleDict1.keys()))
    # dict_keys = list(titleDict1.keys())
    # dict1_values = list(titleDict2.values())
    # dict2_values = list(titleDict2.values())
    # myrange = list(it.product(range(3),range(3)))
    # for range_ in myrange : 
    #     #print(range_)
    #     sent1 = str(dict1_values[range_[0]])
    #     sent2 = str(dict2_values[range_[1]])
    #     if len(sent1.split()) == 0 or len(sent2.split()) == 0 :
    #         return 0
    #     dist = sequenceDistance(sent1, sent2)
    #     if dist <= threshold :
    #         return 1

    #     split1 = [token for token in re.findall("\w+", sent1) if token not in titleStopwords]
    #     split2 = [token for token in re.findall("\w+", sent2) if token not in titleStopwords]
        
    #     if len(split1) == 0 or len(split2) == 0 :
    #         #and (split1[0] in titleStopwords or titleDict2Tokens[0] in titleStopwords):
    #         return 0
    
    # return -1        
    
    for key in sortedKeys :
        for otherKey in sortedKeys :
            titleDict1Tokens = titleDict1[key].split()
            titleDict2Tokens = titleDict2[otherKey].split()
            if len(titleDict1Tokens) == 0 or len(titleDict2Tokens) == 0 :
                return 0

            if len(titleDict1Tokens) == 1 or len(titleDict2Tokens) == 1 \
                and (titleDict1Tokens[0] in titleStopwords or titleDict2Tokens[0] in titleStopwords):
                return 0

            dist = sequenceDistance(titleDict1[key], titleDict2[otherKey])
            if dist <= threshold :
                return 1
    return -1
